<?php
/**
 * $Horde: klutz/comics.php,v 1.58 2005/06/01 03:32:20 selsky Exp $
 *
 * Copyright 2002-2005 Marcus I. Ryan <marcus@riboflavin.net>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

@define('KLUTZ_BASE', dirname(__FILE__));
require_once KLUTZ_BASE . '/lib/base.php';

// Get the name of the comic we want to see, if passed in.
$index = Util::getFormData('index');

// Make sure we have a valid date set.
$date = Util::getFormData('date');
if (!empty($date) && !is_numeric($date)) {
    $date = strtotime($date);
}

// Set default page title and date format.
$title = _("Today's Comics");
$date_format = 'D, d M Y H:i:s T';

// Figure out how many days of history we keep.
$daystokeep = 90;
if (!empty($conf['backend']['daystokeep'])) {
    $daystokeep = $conf['backend']['daystokeep'];
}

// Get/guess the current action ID.
$actionID = Util::getFormData('actionID');
if (is_null($actionID)) {
    if (empty($date) && empty($index)) {
        $actionID = 'main';
    } elseif (empty($date) && !empty($index)) {
        $actionID = 'comic';
    } elseif (!empty($date) && empty($index)) {
        $actionID = 'day';
    } else {
        $actionID = 'image';
    }
}
if (empty($date)) {
    $date = mktime(0, 0, 0);
}

// If we're not displaying a raw image, include the common
// header file and perform some useful date calculations.
if ($actionID != 'image') {
    require KLUTZ_TEMPLATES . '/common-header.inc';
    require KLUTZ_TEMPLATES . '/menu.inc';

    // Figure out timestamps for navigation links.

    $time = mktime(0, 0, 0);
    $t = getdate($time);
    $dates = $storage->listDates($time, mktime(0, 0, 0,
                                               $t['mon'],
                                               $t['mday'] - $daystokeep,
                                               $t['year']));

    if (count($dates) > 0) {
        $first_day = $dates[0];
        $last_day = $dates[count($dates) - 1];
    } else {
        $first_day = null;
        $last_day = null;
    }

    $d = getdate($date);
    $prev_month = mktime(0, 0, 0, $d['mon'] - 1, 1, $d['year']);
    $next_month = mktime(0, 0, 0, $d['mon'] + 1, 1, $d['year']);
    $yesterday = mktime(0, 0, 0, $d['mon'], $d['mday'] - 1, $d['year']);
    $tomorrow = mktime(0, 0, 0, $d['mon'], $d['mday'] + 1, $d['year']);


    $url = Horde::selfUrl(false, false);
    $url = Util::addParameter($url, 'actionID', $actionID);

    $first_day_url = null;
    $prev_month_url = null;
    $yesterday_url = null;
    $tomorrow_url = null;
    $next_month_url = null;
    $last_day_url = null;
}

switch ($actionID) {
case 'image':
    $index = Util::getFormData('index');
    $image = $storage->retrieveImage($index, $date);

    if (is_object($image)) {
        header("Content-type: " . $image->type);
        if ($image->lastmodified > 0) {
            header('Last-Modified: ' .
                   gmdate($date_format, $image->lastmodified));
        }
        header('Expires: ' . gmdate($date_format, time() + 172800));
        header('Cache-Control: public, max-age=172800');
        header('Pragma:');
        print $image->data;
    } elseif (is_string($image) && substr($image, 0, 4) == 'http') {
        header("Location: $image");
        exit();
    } else {
        // Do some kind of error handling here.
    }
    break;

case 'main':
    $title = _("Main Listing");
    if ($prefs->getValue('show_unselected')) {
        $comics = $klutz->listEnabled();
    } else {
        // Get the list of comics to display.
        $comics = explode("\t", $prefs->getValue('viewcomics'));
        if (count($comics) == 1 && empty($comics[0])) {
            $comics = null;
        }
        $comics = $klutz->listEnabled($comics);
    }

    $selected = explode("\t", $prefs->getValue('viewcomics'));
    if (count($selected) == 1 && empty($selected[0])) {
        $selected = $comics;
    }

    $visited = explode("\t", $prefs->getValue('datesvisited'));
    if (count($visited) == 1 && empty($visited[0])) {
        $visited = array();
    }
    if (count($visited) > $daystokeep) {
        sort($visited, SORT_NUMERIC);
        $f = date('Ymd', $first_day);
        // Try the easy way first - find the actual date in the array
        $i = array_search($f, $visited);
        if ($i !== false) {
            // Note that this will remove up to the date searched for, not
            // that date itself
            array_splice($visited, 0, $i);
        } else {
            // guess we have to do it the slow/tedious way
            $c = count($visited);
            for ($i = 0; $i < $c; $i++) {
                if ($visited[$i] >= $f) break;
                unset($visited[$i]);
            }
        }
        $prefs->setValue('datesvisited', implode("\t", $visited));
    }

    $comics = $klutz->getProperty($comics, array('name', 'author'));

    // Date navigation.
    if (count($storage->listDates($prev_month)) > 0) {
        $prev_month_url = Util::addParameter($url, 'date', $prev_month);
    }
    if (count($storage->listDates($next_month)) > 0) {
        $next_month_url = Util::addParameter($url, 'date', $next_month);
    }

    require KLUTZ_TEMPLATES . '/comics/main.inc';
    break;

case 'day':
    $title = strftime('%B %d, %Y', $date);

    // Display the navbar.
    if (in_array($yesterday, $storage->listDates($yesterday))) {
        $yesterday_url = Util::addParameter($url, 'date', $yesterday);
    }
    if (in_array($tomorrow, $storage->listDates($tomorrow))) {
        $tomorrow_url = Util::addParameter($url, 'date', $tomorrow);
    }
    if (date('dmY', $date) != date('dmY', $first_day)) {
        $first_day_url = Util::addParameter($url, 'date', $first_day);
    }
    if (date('dmY', $date) != date('dmY', $last_day)) {
        $last_day_url = Util::addParameter($url, 'date', $last_day);
    }

    require KLUTZ_TEMPLATES . '/comics/nav_bar.inc';

    // Used for tracking dates we've already looked at.
    $today = date('Ymd', $date);

    // Get the list of comics to display.
    $comics = explode("\t", $prefs->getValue('viewcomics'));
    if (count($comics) == 1 && empty($comics[0])) {
        $comics = null;
    }

    // Update prefs to mark this date as visited.
    $visited = explode("\t", $prefs->getValue('datesvisited'));
    if (count($visited) == 1 && empty($visited[0])) {
        $visited = array();
    }
    if (!$prefs->isLocked('datesvisited') && !in_array($today, $visited)) {
        $visited[] = $today;
        $prefs->setValue('datesvisited', implode("\t", $visited));
    }

    foreach ($klutz->listEnabled($comics, $date) as $index) {
        $name = $klutz->getProperty($index, 'name');
        $author = $klutz->getProperty($index, 'author');
        $homepage = $klutz->getProperty($index, 'homepage');
        if ($storage->imageExists($index, $date)) {
            $size = $storage->imageSize($index, $date);
            $url = Horde::selfUrl(false, false);
            $url = Util::addParameter($url, 'actionID', 'image');
            $url = Util::addParameter($url, 'date', $date);
            $url = Util::addParameter($url, 'index', $index);
            require KLUTZ_TEMPLATES . '/comics/comic.inc';
        } elseif ($klutz->getProperty($index, 'days') == 'random') {
            @include KLUTZ_TEMPLATES . '/comics/random.inc';
        } else {
            require KLUTZ_TEMPLATES . '/comics/missing.inc';
        }
    }
    require KLUTZ_TEMPLATES . '/comics/nav_bar.inc';
    break;

case 'comic':
    // Get a list of the available comics the user reads
    $comics = explode("\t", $prefs->getValue('viewcomics'));
    if ((count($comics) == 1 && empty($comics[0]))
        || !in_array($index, $comics)) {
        $comics = null;
    }
    $comics = $klutz->listEnabled($comics);
    $i = array_search($index, $comics);
    if ($i !== false) {
        $first_comic = null;
        $prev_comic = null;
        $next_comic = null;
        $last_comic = null;

        $curl = Util::addParameter($url, 'date', $date);
        if ($comics[0] != $index) {
            $first_comic = Util::addParameter($curl, 'index', $comics[0]);
        }
        if ($i > 0 && $comics[$i-1] != $index) {
            $prev_comic = Util::addParameter($curl, 'index', $comics[$i-1]);
        }
        if ($i < (count($comics) - 1) && $comics[$i+1] != $index) {
            $next_comic = Util::addParameter($curl, 'index', $comics[$i+1]);
        }
        $l = count($comics) - 1;
        if ($comics[$l] != $index) {
            $last_comic = Util::addParameter($curl, 'index', $comics[$l]);
        }
    } else {
        $notification->push(_("This comic doesn't exist or is disabled."),
                            'horde.error');
        $url = Horde::selfUrl(false, false);
        $url = Util::addParameter($url, 'date', $date);
        $url = Util::addParameter($url, 'actionID', 'main');
        header("Location: $url");
    }

    $comic_select = null;
    if (count($comics) > 4) {
        $comic_select = '<select name="index" onchange="this.form.submit()">';
        foreach ($comics as $c) {
            $comic_select .= ' <option value="' . $c . '"';
            if ($c == $index) {
                $comic_select .= ' selected="selected"';
            }
            $comic_select .= '>' . $klutz->getProperty($c, 'name');
            $comic_select .= '</option>\n';
        }
        $comic_select .= '</select>';
    }

    // Display the navbar.
    $url = Util::addParameter($url, 'index', $index);
    if (!is_null($first_day) and date('dmY', $date) != date('dmY', $first_day)) {
        $first_day_url = Util::addParameter($url, 'date', $first_day);
    }
    if (!is_null($last_day) and date('dmY', $date) != date('dmY', $last_day)) {
        $last_day_url = Util::addParameter($url, 'date', $last_day);
    }
    if (count($storage->listDates($prev_month)) > 0) {
        $prev_month_url = Util::addParameter($url, 'date', $prev_month);
    }
    if (count($storage->listDates($next_month)) > 0) {
        $next_month_url = Util::addParameter($url, 'date', $next_month);
    }
    require KLUTZ_TEMPLATES . '/comics/nav_bar.inc';

    $name = $klutz->getProperty($index, 'name');
    $author = $klutz->getProperty($index, 'author');
    $homepage = $klutz->getProperty($index, 'homepage');
    $title = sprintf(_("%s by %s"), $name, $author);
    foreach ($storage->listDates($date) as $date) {
        if ($storage->imageExists($index, $date)) {
            $size = $storage->imageSize($index, $date);
            $url = Horde::selfUrl(false, false);
            $url = Util::addParameter($url, 'actionID', 'image');
            $url = Util::addParameter($url, 'date', $date);
            $url = Util::addParameter($url, 'index', $index);
            require KLUTZ_TEMPLATES . '/comics/comic.inc';
        } elseif ($klutz->getProperty($index, 'days') == 'random') {
            @include KLUTZ_TEMPLATES . '/comics/random.inc';
        } elseif (!in_array(String::lower(date('D', $date)), $klutz->getProperty($index, 'days'))) {
            continue;
        } else {
            include KLUTZ_TEMPLATES . '/comics/missing.inc';
        }
    }
    require KLUTZ_TEMPLATES . '/comics/nav_bar.inc';
    break;
}

if ($actionID != 'image') {
    require $registry->get('templates', 'horde') . '/common-footer.inc';
}
